Stan Math Library: stan/math/prim/scal/prob/pareto_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

pareto_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_PARETO_LOG_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_PARETO_LOG_HPP
3
4 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
5 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
6 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
7 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
8 #include <stan/math/prim/scal/err/check_greater_or_equal.hpp>
9 #include <stan/math/prim/scal/err/check_nonnegative.hpp>
10 #include <stan/math/prim/scal/err/check_not_nan.hpp>
11 #include <stan/math/prim/scal/err/check_positive_finite.hpp>
12 #include <stan/math/prim/scal/fun/constants.hpp>
13 #include <stan/math/prim/scal/fun/multiply_log.hpp>
14 #include <stan/math/prim/scal/fun/value_of.hpp>
15 #include <stan/math/prim/scal/meta/include_summand.hpp>
16 #include <stan/math/prim/scal/meta/contains_nonconstant_struct.hpp>
17 #include <stan/math/prim/scal/meta/VectorBuilder.hpp>
18 #include <boost/random/exponential_distribution.hpp>
19 #include <boost/random/variate_generator.hpp>
20 #include <cmath>
21
22 namespace stan {
23 namespace math {
24
25 // Pareto(y|y_m, alpha) [y > y_m; y_m > 0; alpha > 0]
26 template <bool propto,
27 typename T_y, typename T_scale, typename T_shape>
28 typename return_type<T_y, T_scale, T_shape>::type
29 pareto_log(const T_y& y, const T_scale& y_min, const T_shape& alpha) {
30 static const char* function("stan::math::pareto_log");
31 typedef typename stan::partials_return_type<T_y, T_scale, T_shape>::type
32 T_partials_return;
33
34 using stan::math::value_of;
35 using stan::math::check_positive_finite;
36 using stan::math::check_not_nan;
37 using stan::math::check_consistent_sizes;
38 using std::log;
39
40 // check if any vectors are zero length
41 if (!(stan::length(y)
42 && stan::length(y_min)
43 && stan::length(alpha)))
44 return 0.0;
45
46 // set up return value accumulator
47 T_partials_return logp(0.0);
48
49 // validate args (here done over var, which should be OK)
50 check_not_nan(function, "Random variable", y);
51 check_positive_finite(function, "Scale parameter", y_min);
52 check_positive_finite(function, "Shape parameter", alpha);
53 check_consistent_sizes(function,
54 "Random variable", y,
55 "Scale parameter", y_min,
56 "Shape parameter", alpha);
57
58 // check if no variables are involved and prop-to
59 if (!include_summand<propto, T_y, T_scale, T_shape>::value)
60 return 0.0;
61
62 VectorView<const T_y> y_vec(y);
63 VectorView<const T_scale> y_min_vec(y_min);
64 VectorView<const T_shape> alpha_vec(alpha);
65 size_t N = max_size(y, y_min, alpha);
66
67 for (size_t n = 0; n < N; n++) {
68 if (y_vec[n] < y_min_vec[n])
69 return LOG_ZERO;
70 }
71
72 // set up template expressions wrapping scalars into vector views
73 OperandsAndPartials<T_y, T_scale, T_shape>
74 operands_and_partials(y, y_min, alpha);
75
76 VectorBuilder<include_summand<propto, T_y, T_shape>::value,
77 T_partials_return, T_y> log_y(length(y));
78 if (include_summand<propto, T_y, T_shape>::value) {
79 for (size_t n = 0; n < length(y); n++)
80 log_y[n] = log(value_of(y_vec[n]));
81 }
82
83 VectorBuilder<contains_nonconstant_struct<T_y, T_shape>::value,
84 T_partials_return, T_y> inv_y(length(y));
85 if (contains_nonconstant_struct<T_y, T_shape>::value) {
86 for (size_t n = 0; n < length(y); n++)
87 inv_y[n] = 1 / value_of(y_vec[n]);
88 }
89
90 VectorBuilder<include_summand<propto, T_scale, T_shape>::value,
91 T_partials_return, T_scale>
92 log_y_min(length(y_min));
93 if (include_summand<propto, T_scale, T_shape>::value) {
94 for (size_t n = 0; n < length(y_min); n++)
95 log_y_min[n] = log(value_of(y_min_vec[n]));
96 }
97
98 VectorBuilder<include_summand<propto, T_shape>::value,
99 T_partials_return, T_shape> log_alpha(length(alpha));
100 if (include_summand<propto, T_shape>::value) {
101 for (size_t n = 0; n < length(alpha); n++)
102 log_alpha[n] = log(value_of(alpha_vec[n]));
103 }
104
105 using stan::math::multiply_log;
106
107 for (size_t n = 0; n < N; n++) {
108 const T_partials_return alpha_dbl = value_of(alpha_vec[n]);
109 // log probability
110 if (include_summand<propto, T_shape>::value)
111 logp += log_alpha[n];
112 if (include_summand<propto, T_scale, T_shape>::value)
113 logp += alpha_dbl * log_y_min[n];
114 if (include_summand<propto, T_y, T_shape>::value)
115 logp -= alpha_dbl * log_y[n] + log_y[n];
116
117 // gradients
118 if (!is_constant_struct<T_y>::value)
119 operands_and_partials.d_x1[n] -= alpha_dbl * inv_y[n] + inv_y[n];
120 if (!is_constant_struct<T_scale>::value)
121 operands_and_partials.d_x2[n] += alpha_dbl / value_of(y_min_vec[n]);
122 if (!is_constant_struct<T_shape>::value)
123 operands_and_partials.d_x3[n]
124 += 1 / alpha_dbl + log_y_min[n] - log_y[n];
125 }
126 return operands_and_partials.value(logp);
127 }
128
129 template <typename T_y, typename T_scale, typename T_shape>
130 inline
131 typename return_type<T_y, T_scale, T_shape>::type
132 pareto_log(const T_y& y, const T_scale& y_min, const T_shape& alpha) {
133 return pareto_log<false>(y, y_min, alpha);
134 }
135 }
136 }
137 #endif
check_not_nan.hpp
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
stan::math::check_not_nan
bool check_not_nan(const char *function, const char *name, const T_y &y)
Return true if y is not NaN.
Definition: check_not_nan.hpp:63
check_positive_finite.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
stan::math::log
fvar< T > log(const fvar< T > &x)
Definition: log.hpp:15
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
contains_nonconstant_struct.hpp
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
stan::math::pareto_log
return_type< T_y, T_scale, T_shape >::type pareto_log(const T_y &y, const T_scale &y_min, const T_shape &alpha)
Definition: pareto_log.hpp:29
is_constant_struct.hpp
stan::math::LOG_ZERO
const double LOG_ZERO
Definition: constants.hpp:175
stan::math::include_summand
Template metaprogram to calculate whether a summand needs to be included in a proportional (log) prob...
Definition: include_summand.hpp:36
stan::return_type::type
boost::math::tools::promote_args< typename scalar_type< T1 >::type, typename scalar_type< T2 >::type, typename scalar_type< T3 >::type, typename scalar_type< T4 >::type, typename scalar_type< T5 >::type, typename scalar_type< T6 >::type >::type type
Definition: return_type.hpp:27
include_summand.hpp
multiply_log.hpp
check_nonnegative.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
stan::math::OperandsAndPartials::d_x3
VectorView< T_return_type, false, true > d_x3
Definition: OperandsAndPartials.hpp:41
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
VectorBuilder.hpp
partials_return_type.hpp
OperandsAndPartials.hpp
stan::contains_nonconstant_struct
Definition: contains_nonconstant_struct.hpp:14
stan::math::multiply_log
fvar< T > multiply_log(const fvar< T > &x1, const fvar< T > &x2)
Definition: multiply_log.hpp:15
stan::VectorBuilder
VectorBuilder allocates type T1 values to be used as intermediate values.
Definition: VectorBuilder.hpp:28
constants.hpp
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
check_greater_or_equal.hpp
value_of.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
stan::partials_return_type::type
boost::math::tools::promote_args< typename partials_type< typename scalar_type< T1 >::type >::type, typename partials_type< typename scalar_type< T2 >::type >::type, typename partials_type< typename scalar_type< T3 >::type >::type, typename partials_type< typename scalar_type< T4 >::type >::type, typename partials_type< typename scalar_type< T5 >::type >::type, typename partials_type< typename scalar_type< T6 >::type >::type >::type type
Definition: partials_return_type.hpp:26
check_consistent_sizes.hpp
stan::math::check_positive_finite
bool check_positive_finite(const char *function, const char *name, const T_y &y)
Return true if y is positive and finite.
Definition: check_positive_finite.hpp:28
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39

[ Stan Home Page ] © 2011–2016, Stan Development Team.